using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace Ilungasoft.Helper.ObjectPooling
{
    internal class SyncGuard
    {
        #region Private Members

        private object target;
        private bool locked;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="T:SyncGuard"/> class.
        /// </summary>
        /// <param name="target">The target.</param>
        public SyncGuard(object target)
        {
            locked = false;
            if (target == null)
            {
                throw new ArgumentNullException("target");
            }
            this.target = target;
        }

        /// <summary>
        /// Releases unmanaged resources and performs other cleanup operations before the
        /// <see cref="T:Ilungasoft.Helper.ObjectPooling.SyncGuard"/> is reclaimed by garbage collection.
        /// </summary>
        ~SyncGuard()
        {
            RemoveLocks();
        }

        #endregion

        #region Public Members

        /// <summary>
        /// Locks this instance.
        /// </summary>
        public void Lock()
        {
            Monitor.Enter(target);
            locked = true;
        }

        /// <summary>
        /// Unlocks this instance.
        /// </summary>
        public void Unlock()
        {
            Monitor.Exit(target);
            locked = false;
        }

        /// <summary>
        /// Removes the locks.
        /// </summary>
        public void RemoveLocks()
        {
            if (locked)
            {
                Unlock();
            }
        }

        #endregion
    }
}
